#include <Wire.h>
#include <Adafruit_ADS1015.h>
#include <Ethernet.h>

byte mac[] = {0x00, 0xAA, 0xBB, 0xCC, 0xDE, 0x02};
byte machServer[] = {192, 168, 1, 134};
int port = 5000;

EthernetServer server(port);
EthernetClient client;

Adafruit_ADS1115 ads(0x48);
float voltage = 0.0;

int adc0;
int lastADCSample;
int adcSample;
int numSamples = 50;

void setup() {
  Ethernet.init(53); 
  Serial.begin(9600);
  ads.setGain(GAIN_TWOTHIRDS);  // 2/3x gain +/- 6.144V  1 bit = 3mV      0.1875mV (default)
  ads.begin();

  // start the Ethernet connection:
  Serial.println("Initialize Ethernet with DHCP:");
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP");
    if (Ethernet.hardwareStatus() == EthernetNoHardware) {
      Serial.println("Ethernet shield was not found.  Sorry, can't run without hardware. :(");
    } else if (Ethernet.linkStatus() == LinkOFF) {
      Serial.println("Ethernet cable is not connected.");
    }
    // no point in carrying on, so do nothing
    while (true) {
      delay(1);
    }
  }
  // print your local IP address:
  Serial.print("My IP address: ");
  Serial.println(Ethernet.localIP());

  Serial.println("Commands: \n c to connect\n x to disconnect");
  //server.begin();
}

int filterADC() {
  numSamples--;
  adcSample += ads.readADC_SingleEnded(0);

  if (numSamples == 0) {
    numSamples = 50;
    adcSample = adcSample / numSamples; 
  }
  
  return adcSample;   
}

void loop() {
  if (client.connected()) {
    adcSample = 0;
    adc0 = filterADC();

    if (adc0 == lastADCSample or adc0 < 6500) {
      return;
    }

    lastADCSample = adc0;
    
    Serial.println(adc0);
    voltage = (adc0 * 0.1875)/1000;

    // message type
    int d = 101 + '0';
    char c = d - '0';
    client.println(c);
    delay(10);

    // message data
    client.println(voltage);
    //Serial.println(voltage);
    delay(100);  
  }
  
  readSerialInput();
}

void readSerialInput() {
  if (Serial.available() > 0) {
    char incomingByte = Serial.read();
    Serial.println(incomingByte);
    
    switch(incomingByte) {
      case 'x': 
        Serial.println();
        Serial.println("Goodbye Server");
        delay(250);
        client.stop();
        break;
      case 'c': 
        if (client.connect(machServer, port)) {
          Serial.println("connected to server");
        } else {
          Serial.println("unable to connect to the server");
        }
        break;
    }
  }
}
